<?php

namespace onespace\tools\activeApi\components\clients\teams;

/**
 * Enum of loggable OneSpace microservices which has a teams logging channel
 */
enum OneSpaceService {
    case ONEACCESS;
    case ONEDEVICE;
    case AUTH;
    case CONNECT;
    case CENTRAL;

    /** This is not a service, but to be used when doing testing and dev work */
    case DEVELOPMENT;
}
